@@ -269,7 +269,7 @@ void CImage::writeSplit()
   catch ( CExceptions * excep )  // exception = errno
     { qwFreeSpace = 0LL; }*/
       
-  m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0);
+  m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0, m_options.szFullyBatchMode);
   showDebug(2, "imagefile split done\n");
 
   RETURN;
@@ -315,7 +315,8 @@ void CImage::write(void *vBuf, DWORD dwLength, bool bUpdateCRC)
       catch ( CExceptions * excep )  // exception = errno
 	{ qwFreeSpace = 0LL; }
       
-      m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0);
+      m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, 0, m_options.szFullyBatchMode);
+      qwTotalBytes = 0LL;
       showDebug(2, "imagefile split done\n");
     }
   
@@ -356,7 +357,7 @@ void CImage::write(void *vBuf, DWORD dwLength, bool bUpdateCRC)
 	{ qwFreeSpace = 0LL; }
 
       if (m_guiSave)
-	m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, get_qwCurrentVolumeSize());  
+          m_guiSave -> showImageFileInfo(get_szImageFilename(), qwFreeSpace, get_qwCurrentVolumeSize(), m_options.szFullyBatchMode);
     }
 
   qwCount += (QWORD) dwRes;
@@ -420,13 +421,17 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
 	    }
           qwTotalBytes = 0LL;
 	  read(cBuf+dwFirstRead, dwLength-dwFirstRead, bUpdateCRC);
-	  m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression);
-	}
-      else
+          m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression, m_options.szFullyBatchMode);
+        }
+      else if ((g_nThreadState == THREAD_ASKEXIT) || (g_nThreadState == THREAD_FINISHED) )
 	{
 	  showDebug(1, "READ ERROR 2: dwLength=%lu and dwRes=%lu and g_nThreadState=%d\n", dwLength, dwRes, g_nThreadState);
-	  THROW(ERR_READING);
 	}
+      else
+      {
+          showDebug(1, "READ ERROR 2: dwLength=%lu and dwRes=%lu and g_nThreadState=%d\n", dwLength, dwRes, g_nThreadState);
+          THROW(ERR_READING);
+      }
     }
 
   qwTotalBytes += (QWORD) dwLength;
@@ -437,7 +442,7 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
     {
       qwCount = 0LL;
       if (m_guiRestore)
-	m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression);
+        m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression, m_options.szFullyBatchMode);
     }
 
   if (dwLength != 1)
@@ -470,7 +475,7 @@ void CImage::splitChangeImageFileWrite()
       if (!fork()) // we're the child // no error check maid
         execl("/tmp/partimage-shell", 
            get_szImageFilename(), get_szPath(),
-           get_szOriginalFilename(), get_dwVolumeNumber());
+           get_szOriginalFilename(), get_dwVolumeNumber(),(char*)NULL);
     }
 
   // wait for a keyboard keypress if need
@@ -483,7 +488,7 @@ void CImage::splitChangeImageFileWrite()
     }
 
   set_dwVolumeNumber(get_dwVolumeNumber()+1); // increase the volume number
-  SNPRINTF(aux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+  SNPRINTF(aux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
   //*(aux+MAXPATHLEN-1) = '\0';
   
   set_szImageFilename(aux);
@@ -509,7 +514,7 @@ void CImage::splitChangeImageFileRead()
 
   // increase volume number
   set_dwVolumeNumber(get_dwVolumeNumber()+1);
-  SNPRINTF(aux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+  SNPRINTF(aux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
   //*(aux+MAXPATHLEN-1) = '\0';
   set_szImageFilename(aux);
 
@@ -695,7 +700,7 @@ int CImage::openWritingFdDisk()
 	      set_szPath(aux2);
 	      extractFilenameFromFullPath(szAux, aux2); // filename without path
 	      set_szOriginalFilename(aux2);
-	      SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+	      SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
 	      set_szImageFilename(szAux);
 	      nRes = openWritingFdDisk();
 	      RETURN_int(nRes);
@@ -723,7 +728,7 @@ int CImage::openWritingFdDisk()
             case ERR_RETRY:
               strcpy(szAux, excep->getNewString()); // endding '/' were removed 
               set_szPath(szAux); 
-	      SNPRINTF(aux2, "%s/%s.%.3ld", szAux, get_szOriginalFilename(),
+	      SNPRINTF(aux2, "%s/%s.%.3d", szAux, get_szOriginalFilename(),
 		       get_dwVolumeNumber());
               set_szImageFilename(aux2);
 	      
@@ -1033,7 +1038,7 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
   m_options.dwCompression = getCompressionLevelForImage(m_cid->get_szImageFilename());
   showDebug(1, "LEVEL [%s]=%d\n", m_cid->get_szImageFilename(), m_options.dwCompression);
   if (m_guiRestore)
-    m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression);
+    m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression, m_options.szFullyBatchMode);
 
   // ---- MULTI-THREAD INITIALIZATION
   // Run the thread which read the buffer from the image file
@@ -1063,7 +1068,7 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
             set_szPath(aux2);
             extractFilenameFromFullPath(szAux, aux2);
             set_szOriginalFilename(aux2);
-            SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), 
+            SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), 
                get_szOriginalFilename(), get_dwVolumeNumber());
             set_szImageFilename(szAux);
             umountImageLocation();
@@ -1127,7 +1132,7 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
 	
   if (strcmp(headVolume.szMagicString, MAGIC_BEGIN_VOLUME) != 0)
     {	
-      SNPRINTF(szAux, "%s/%s.%.3ld", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
+      SNPRINTF(szAux, "%s/%s.%.3d", get_szPath(), get_szOriginalFilename(), get_dwVolumeNumber());
       THROW(ERR_NOTAPARTIMAGEFILE, szAux);
     }
 
