@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.70 $
+// $Revision: 1.3 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -29,6 +29,10 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
+
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
+
 #include <sched.h>
 #include <sys/types.h>
 #include <sys/param.h>
@@ -103,6 +107,8 @@ static struct option const long_options[] =
 };
 #endif
 
+static char finish_last_countfile_lock[]="/tmp/partimageXXXXXX.lock";
+static char finish_last_countfile[]="/tmp/partimageXXXXXX";
 static char optstring[]="z:oV:ecmdhf:s:p:bwg:vynSMa:iU:P:XB:";
 FILE * g_fDebug; // debug file
 FILE * g_fLocalDebug; // debug file
@@ -133,6 +139,12 @@ static void catch_sigint(int signo)
     }*/
 }
 
+void ensure_interface_is_non_interactive()
+{
+  delete g_interface;
+  g_interface = new CInterfaceNone("");
+}
+
 // =======================================================
 int main(int argc, char *argv[])
 {
@@ -142,6 +154,7 @@ int main(int argc, char *argv[])
   int nChoice;
   char szDevice[MAX_DEVICENAMELEN];
   char szImageFile[MAXPATHLEN];
+  char szImageDir[MAXPATHLEN];
   char szTemp[2048];
   char szTemp2[1024];
   char szTemp3[1024]; 
@@ -175,7 +188,7 @@ int main(int argc, char *argv[])
   options.bSync = OPTIONS_DEFAULT_SYNC;
   options.dwDebugLevel = DEFAULT_DEBUG_LEVEL; // defined by configure.in
   strncpy(options.szServerName, OPTIONS_DEFAULT_SERVERNAME, MAX_HOSTNAMESIZE);
-  strncpy(options.szAutoMount, OPTIONS_DEFAULT_AUTOMOUNT, MAXPATHLEN);
+  strncpy(options.szFullyBatchMode, OPTIONS_DEFAULT_FULLYBATCHMODE, 2048);
   strncpy(options.szUserName, OPTIONS_DEFAULT_USERNAME, MAX_USERNAMELEN);
   strncpy(options.szPassWord, OPTIONS_DEFAULT_PASSWORD, MAX_PASSWORDLEN);
   *(options.szFullyBatchMode) = 0;
@@ -274,8 +287,8 @@ int main(int argc, char *argv[])
 	  
 	  case 'V': // split image into multiple files
 	  
-	    options.qwSplitSize = atol(optarg) * 1024 * 1024;
-	    fprintf (stderr, i18n("Volume size: %llu bytes (%ld MiB)\n"),
+	    options.qwSplitSize = (QWORD)atol(optarg) * 1024 * 1024;
+	    fprintf (stderr, i18n("Volume size: %"PRIu64" bytes (%ld MiB)\n"),
 		     options.qwSplitSize, atol(optarg));
 	    break;
 	  
@@ -505,13 +518,17 @@ int main(int argc, char *argv[])
   if (argc - optind == 2) // commands with 1 parameter
     {
       showDebug(8, "full cmdline with 1 param\n"); 
-      strncpy(szImageFile, argv[optind+1], MAXPATHLEN);
-      
       if (strcmp(argv[optind], "restmbr")==0) // restore an MBR
 	nChoice = OPERATION_RESTMBR;
       else if (strcmp(argv[optind], "imginfo")==0)
 	// show informations about the imagefile
 	nChoice = OPERATION_IMGINFO;
+      else if (strcmp(argv[optind], "save_all")==0)
+	nChoice = OPERATION_SAVE_ALL;
+      else if (strcmp(argv[optind], "rest_all")==0)
+	nChoice = OPERATION_REST_ALL;
+
+      strncpy(nChoice == OPERATION_REST_ALL || nChoice == OPERATION_SAVE_ALL ? szImageDir : szImageFile, argv[optind+1], MAXPATHLEN);      
     }
   
   // check options
@@ -568,8 +585,9 @@ int main(int argc, char *argv[])
 		  goto beginMainWin;
 		}
 
-	      if (!options.bBatchMode)
-		g_interface -> Error(excep, szImageFile, szDevice);
+	      if (options.bBatchMode) ensure_interface_is_non_interactive();
+
+              g_interface -> Error(excep, szImageFile, szDevice);
 
 	      showDebug(1, "\nFINAL ERROR\n\n");
 	  
@@ -580,8 +598,11 @@ int main(int argc, char *argv[])
       else
 	{
 	  showDebug(1, "The file system of [%s] is [%s], and is not supported\n", szDevice, szFileSystem);
-	  if (!options.bBatchMode)
-	    g_interface->msgBoxError("The file system of [%s] is [%s], and is not supported", szDevice, szFileSystem);
+
+          if (options.bBatchMode) ensure_interface_is_non_interactive();
+
+          g_interface->msgBoxError("The file system of [%s] is [%s], and is not supported", szDevice, szFileSystem);
+
 	  nRes = -1;
 	}
 //#endif // DEVEL_SUPPORT
@@ -644,11 +665,48 @@ int main(int argc, char *argv[])
 	  nRes = -1;
 	}
       break;
+
+    case OPERATION_SAVE_ALL:
+      showDebug(1, "action=SAVE_ALL\n");
+      try { save_all(szImageDir, &options); }
+      catch (CExceptions *excep)
+	{
+          showDebug(1, "caught exception: %d\n", excep->GetExcept());
+
+	  if (!options.bBatchMode && !excep->getCaught())
+	    g_interface -> Error(excep, szImageFile);
+
+	  showDebug(1, "\nFINAL ERROR\n\n");
+
+	  nRes = -1;
+	}      
+      break;
+
+    case OPERATION_REST_ALL:
+      showDebug(1, "action=REST_ALL\n");
+      try { rest_all(szImageDir, &options); }
+      catch (CExceptions *excep)
+	{
+          showDebug(1, "caught exception: %d\n", excep->GetExcept());
+
+	  if (!options.bBatchMode && !excep->getCaught())
+	    g_interface -> Error(excep, szImageFile);
+
+	  showDebug(1, "\nFINAL ERROR\n\n");
+
+	  nRes = -1;
+	}      
+      break;
       
     default: // exit
+      if (options.bBatchMode)
+             fprintf(stderr,"\n\n");
+
       if (options.bSync)
         {
-          g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
+          if (options.bBatchMode)
+               fprintf(stderr,"\n\n");
+          if (!options.bBatchMode) g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
 	  sync();
         }
       closeDebugFiles();
@@ -661,6 +719,8 @@ int main(int argc, char *argv[])
   // update the disk IO
   if (options.bSync)
     {
+      if (options.bBatchMode)
+             fprintf(stderr,"\n\n");
       g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
       sync();
     }
@@ -744,6 +804,13 @@ int main(int argc, char *argv[])
       else if (options.dwFinish == FINISH_REBOOT)
 	{
 	  showDebug(1, "option FINISH_REBOOT\n");
+          if (options.bSimulateMode)
+                formatOperationSucessMsg(szTemp, sizeof(szTemp), i18n("Simulation"));
+           else
+               formatOperationSucessMsg(szTemp, sizeof(szTemp), i18n("Operation"));
+          if (nChoice != OPERATION_IMGINFO)
+            g_interface->msgBoxOk(i18n("Success"), szTemp);
+
 	  delete g_interface; 
 	  nRes = system("/sbin/shutdown -r now");
           nRes = system("/sbin/reboot");
@@ -760,68 +827,83 @@ int main(int argc, char *argv[])
       else if (options.dwFinish == FINISH_LAST)
 	{
 	  showDebug(1, "option FINISH_LAST\n");
-	  int nLockFile;
-	  FILE * nCountFile;
-	  char szValue[11];
+	  int fd_lock, fd_countfile;
+ 	  FILE *fCountFile;
+	  FILE *fLockFile;
 	  int nValue, nRetries;
-	  ssize_t nRes;
 
 	  nRetries = 5;
-          do {
-	    nLockFile = open(FINISH_LAST_COUNTFILE_LOCK, O_CREAT|O_EXCL,
-               O_RDONLY);
-	    if (nLockFile == -1)
+          do 
+	  {
+	    if ((fd_lock = mkstemp(finish_last_countfile_lock)) == -1)
 	      {
-                --nRetries;
-		sleep(2);
+		delete g_interface;
+		fprintf(stderr, i18n("Error: Cannot create lock for countfile"));
+		return EXIT_FAILURE;
 	      }
-	  } while (nLockFile == -1 && nRetries);
-	  if (!nRetries)
+	    fLockFile = fdopen(fd_lock, "r");
+	    if (fLockFile == NULL)
+ 	      {
+                 --nRetries;
+ 		sleep(2);
+ 	      }
+	  } while (fLockFile == NULL && nRetries);
+ 	  if (!nRetries)
+ 	    {
+    	      delete g_interface;
+ 	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
+ 	      return EXIT_SUCCESS;
+ 	    }
+	  if ((fd_countfile = mkstemp(finish_last_countfile)) == -1)
+	    {
+	      delete g_interface;
+	      fprintf(stderr, i18n("Error: Cannot create countfile"));
+	      return EXIT_FAILURE;
+	    }
+          fCountFile = fdopen(fd_countfile, "r");
+	  if (fCountFile == NULL)
 	    {
     	      delete g_interface; 
 	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
 	      return EXIT_SUCCESS;
 	    }
-          nCountFile = fopen(FINISH_LAST_COUNTFILE, "r");
-	  if (nCountFile == NULL)
-	    {
-    	      delete g_interface; 
-	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
-	      return EXIT_SUCCESS;
-	    }
-	  fscanf(nCountFile, "%d\n", &nValue);
+	  fscanf(fCountFile, "%d\n", &nValue);
 	  showDebug(1, "RESTANT: %d\n", nValue);
 	  if (nValue <= 1)
-	    { // we are the last running partimage -> shutdown
-	      delete g_interface; 
-	      fclose(nCountFile);
-	      close(nLockFile);
-	      unlink(FINISH_LAST_COUNTFILE_LOCK);
-	      unlink(FINISH_LAST_COUNTFILE);
-//	      nRes = system("/sbin/shutdown -r now");
-//              nRes = system("/sbin/reboot");
-// if we reach this point, it's because shutdown failed
+ 	    { // we are the last running partimage -> shutdown
+ 	      delete g_interface; 
+ 	      fclose(fCountFile);
+	      fclose(fLockFile);
+	      close(fd_lock);
+	      close(fd_countfile);
+	      unlink(finish_last_countfile_lock);
+	      unlink(finish_last_countfile);
+ //	      nRes = system("/sbin/shutdown -r now");
+ //              nRes = system("/sbin/reboot");
+ // if we reach this point, it's because shutdown failed
 	      fprintf(stderr, i18n("DEBUG: reboot the computer"));
 	      return EXIT_SUCCESS;
 	    }
 	  else
 	    {
               --nValue;
-	      fclose(nCountFile);
-              nCountFile = fopen(FINISH_LAST_COUNTFILE, "w");
-	      if (nCountFile == NULL)
+	      fclose(fCountFile);
+              fCountFile = fdopen(fd_countfile, "w");
+	      if (fCountFile == NULL)
 	        {
     	          delete g_interface; 
 	          fprintf(stderr, i18n("Error: Cannot count remaining "
                      "partimages"));
 	          return EXIT_SUCCESS;
 	        }
-	      fprintf(nCountFile, "%d\n", nValue);
-	      fclose(nCountFile);
-	      close(nLockFile);
-	      unlink(FINISH_LAST_COUNTFILE_LOCK);
-    	      delete g_interface; 
-	      return EXIT_SUCCESS;
+	      fprintf(fCountFile, "%d\n", nValue);
+	      fclose(fCountFile);
+	      fclose(fLockFile);
+	      close(fd_lock);
+	      close(fd_countfile);
+	      unlink(finish_last_countfile_lock);
+     	      delete g_interface; 
+ 	      return EXIT_SUCCESS;
 	    }
         }
     }
@@ -842,7 +924,8 @@ void usage()
   printf("===============================================================================\n");
   printf(i18n("Partition Image (http://www.partimage.org/) version %s [%s]\n"
 	      "---- distributed under the GPL 2 license (GNU General Public License) ----\n\n"
-	      "Supported file systems:....ext2fs, reiserfs, fat16, fat32, ntfs(exp), hpfs\n\n"
+	      "Supported file systems: Ext2/3, ReiserFS, FAT16/32, HPFS, JFS, XFS,\n"
+	      "                        UFS(beta), HFS(beta), NTFS(experimental)\n\n"
 	      "usage: partimage [options] <action> <device> <image_file>\n"
 	      "       partimage <imginfo/restmbr> <image_file>\n\n"
 	      "ex: partimage -z1 -o -d save /dev/hda12 /mnt/backup/redhat-6.2.partimg.gz\n"
@@ -858,39 +941,42 @@ void usage()
 	      "  - restmbr: restore a MBR of the image file to an hard disk\n"
 	      "  - imginfo: show informations about the image file\n"
 	      "* <device>: partition to save/restore (example: /dev/hda1)\n"
-	      "* <image_file>: file where data will be read/written. can be very big\n\n"
+	      "* <image_file>: file where data will be read/written. Can be very big.\n"
+	      "                For restore, <image_file> can have the value 'stdin'. This allows\n"
+              "                for providing image files through a pipe.\n\n"
 	      "Options:\n"
-	      "* -z,  --compress     (image file compression level):\n"
-	      "  -z0, --compress=0   don't compress: very fast but very big image file\n"
-	      "  -z1, --compress=1   compress using gzip: fast and small image file (default)\n"
-	      "  -z2, --compress=2   (compress using bzip2: very slow and very small image file):\n"
-	      "* -c,  --nocheck      don't check the partition before saving\n"
-	      "* -o,  --overwrite    overwrite the existing image file without confirmation\n"
-	      "* -d,  --nodesc       don't ask any description for the image file\n"
-	      "* -V,  --volume       (split image into multiple volumes files)\n"
-	      "  -VX, --volume=X     create volumes with a size of X KB\n"
-	      "* -w,  --waitvol      wait for a confirmation after each volume change\n"
-	      "* -e,  --erase        erase empty blocks on restore with zero bytes\n"
-	      "* -m,  --allowmnt     don't fail if the partition is mounted. Dangerous !\n"
-	      "* -M,  --nombr        don't create a backup of the MBR (Mast Boot Record) in the image file\n"
-	      "* -h,  --help         show help\n"
-	      "* -v,  --version      show version\n"
-	      "* -i,  --compilinfo   show compilation options used\n"
-	      "* -f,  --finish       (action to do if finished successfully):\n"
-	      "  -f0, --finish=0     wait: don't make anything\n"
-	      "  -f1, --finish=1     halt (power off) the computer\n"
-	      "  -f2, --finish=2     reboot (restart the computer):\n"
-	      "  -f3, --finish=3     quit\n"
-	      "* -b,  --batch        batch mode: the GUI won't wait for an user action\n"
-	      "* -y,  --nosync       don't synchronize the disks at the end of the operation (dangerous)\n"
-	      "* -sX, --server=X     give partimaged server's ip address\n"
-	      "* -pX, --port=X       give partimaged server's listening port\n"
-	      "* -g,  --debug=X      set the debug level to X (default: 1):\n"
-	      "* -n,  --nossl        disable SSL in network mode\n"
-	      "* -S,  --simulate     simulation of restoration mode\n"
-	      "* -aX, --automnt=X    Automatic mount with X options. Read the doc for more details\n"
-		  "* -UX  --username=X   Username to authenticate to server\n"
-		  "* -PX  --Password=X   Password for authentication of user to server\n"),
+	      "* -z,  --compress      (image file compression level):\n"
+	      "  -z0, --compress=0    don't compress: very fast but very big image file\n"
+	      "  -z1, --compress=1    compress using gzip: fast and small image file (default)\n"
+	      "  -z2, --compress=2    (compress using bzip2: very slow and very small image file):\n"
+	      "* -c,  --nocheck       don't check the partition before saving\n"
+	      "* -o,  --overwrite     overwrite the existing image file without confirmation\n"
+	      "* -d,  --nodesc        don't ask any description for the image file\n"
+	      "* -V,  --volume        (split image into multiple volumes files)\n"
+	      "  -VX, --volume=X      create volumes with a size of X MB\n"
+	      "* -w,  --waitvol       wait for a confirmation after each volume change\n"
+	      "* -e,  --erase         erase empty blocks on restore with zero bytes\n"
+	      "* -m,  --allowmnt      don't fail if the partition is mounted. Dangerous !\n"
+	      "* -M,  --nombr         don't create a backup of the MBR (Mast Boot Record) in the image file\n"
+	      "* -h,  --help          show help\n"
+	      "* -v,  --version       show version\n"
+	      "* -i,  --compilinfo    show compilation options used\n"
+	      "* -f,  --finish        (action to do if finished successfully):\n"
+	      "  -f0, --finish=0      wait: don't make anything\n"
+	      "  -f1, --finish=1      halt (power off) the computer\n"
+	      "  -f2, --finish=2      reboot (restart the computer):\n"
+	      "  -f3, --finish=3      quit\n"
+	      "* -b,  --batch         batch mode: the GUI won't wait for an user action\n"
+              "* -BX, --fully-batch=X batch mode without GUI, X is a challenge response string\n"
+	      "* -y,  --nosync        don't synchronize the disks at the end of the operation (dangerous)\n"
+	      "* -sX, --server=X      give partimaged server's ip address\n"
+	      "* -pX, --port=X        give partimaged server's listening port\n"
+	      "* -g,  --debug=X       set the debug level to X (default: 1):\n"
+	      "* -n,  --nossl         disable SSL in network mode\n"
+	      "* -S,  --simulate      simulation of restoration mode\n"
+	      "* -aX, --automnt=X     automatic mount with X options. Read the doc for more details\n"
+	      "* -UX  --username=X    username to authenticate to server\n"
+	      "* -PX  --password=X    password for authentication of user to server\n"),
 	 PACKAGE_VERSION, isVersionStable(PACKAGE_VERSION) ? i18n("stable") : i18n("unstable")); 
   printf("===============================================================================\n");
 }
@@ -900,15 +986,15 @@ int checkStructSizes()
 {
    // ---- check types sizes
 
-   if (sizeof(unsigned long int) != 4)
+   if (sizeof(DWORD) != 4)
    {
-      fprintf (stderr, "Error: sizeof(DWORD) != 4 (%d)\n", sizeof(unsigned long int));
+      fprintf (stderr, "Error: sizeof(DWORD) != 4 (%zd)\n", sizeof(DWORD));
       goto errcheck;
    }
 
-   if (sizeof(unsigned long long int) != 8)
+   if (sizeof(QWORD) != 8)
    {
-      fprintf (stderr, "Error: sizeof(QWORD) != 8 (%d)\n", sizeof(unsigned long long int));
+      fprintf (stderr, "Error: sizeof(QWORD) != 8 (%zd)\n", sizeof(QWORD));
       goto errcheck;
    }
 
@@ -917,7 +1003,7 @@ int checkStructSizes()
    // check main header size is 16384
    if (sizeof(CMainHeader) != 16384)
    {
-      fprintf (stderr, "Error: main header size != 16384 (%d)\n",
+      fprintf (stderr, "Error: main header size != 16384 (%zd)\n",
 	       sizeof(CMainHeader));
       goto errcheck;
    }
@@ -925,7 +1011,7 @@ int checkStructSizes()
    // check ext2 header size is 16384
    if (sizeof(CInfoExt2Header) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: ext2 header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: ext2 header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoExt2Header));
       goto errcheck;
    }
@@ -933,7 +1019,7 @@ int checkStructSizes()
    // check reiserfs header size is 16384
    if (sizeof(CInfoReiserHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: reiserfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: reiserfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoReiserHeader));
       goto errcheck;
    }
@@ -941,7 +1027,7 @@ int checkStructSizes()
    // check fat header size is 16384
    if (sizeof(CInfoFatHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: fat header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: fat header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoFatHeader));
       goto errcheck;
    }
@@ -949,7 +1035,7 @@ int checkStructSizes()
    // check ntfs header size is 16384
    if (sizeof(CInfoNtfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: ntfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: ntfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoNtfsHeader));
       goto errcheck;
    }
@@ -957,7 +1043,7 @@ int checkStructSizes()
    // check hpfs header size is 16384
    if (sizeof(CInfoHpfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: hpfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: hpfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoHpfsHeader));
       goto errcheck;
    }
@@ -965,7 +1051,7 @@ int checkStructSizes()
    // check jfs header size is 16384
    if (sizeof(CInfoJfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: jfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoJfsHeader));
       goto errcheck;
    }
@@ -973,7 +1059,7 @@ int checkStructSizes()
    // check xfs header size is 16384
    if (sizeof(CInfoXfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: xfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: xfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoXfsHeader));
       goto errcheck;
    }
@@ -981,7 +1067,7 @@ int checkStructSizes()
    // check hfs header size is 16384
    if (sizeof(CInfoHfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: hfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
+      fprintf (stderr, "Error: hfs header size != %d (%zd)\n", INFOS_STRUCT_SIZE,
 	       sizeof(CInfoHfsHeader));
       goto errcheck;
    }
@@ -989,36 +1075,39 @@ int checkStructSizes()
    // check ufs header size is 16384
    if (sizeof(CInfoUfsHeader) != INFOS_STRUCT_SIZE)
    {	
-      fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-	       sizeof(CInfoUfsHeader));
+      fprintf (stderr, "Error: jfs header size != %d (%zd)\n",
+     	 INFOS_STRUCT_SIZE, sizeof(CInfoUfsHeader));
       goto errcheck;
    }
  
    // check tail size is 16384
    if (sizeof(CMainTail) != 16384)
    {	
-      fprintf (stderr, "Error: main footer size != 16384 (%d)\n", sizeof(CMainTail));
+      fprintf (stderr, "Error: main footer size != 16384 (%zd)\n",
+		sizeof(CMainTail));
       goto errcheck;
    }
   
    // check volume header size is 512
    if (sizeof(CVolumeHeader) != 512)
    {	
-      fprintf (stderr, "Error: volume hedaer size != 512 (%d)\n", sizeof(CVolumeHeader));
+      fprintf (stderr, "Error: volume hedaer size != 512 (%zd)\n",
+     	 sizeof(CVolumeHeader));
       goto errcheck;
    }
  
    // check MBR size is 2048
    if (sizeof(CMbr) != 2048)
    {	
-      fprintf (stderr, "Error: MBR size != 2048 (%d)\n", sizeof(CMbr));
+      fprintf (stderr, "Error: MBR size != 2048 (%zd)\n", sizeof(CMbr));
       goto errcheck;
    }
  
    // check volume header size is 512
    if (sizeof(CLocalHeader) != 16384)
    {	
-      fprintf (stderr, "Error: local header size != 16384 (%d)\n", sizeof(CLocalHeader));
+      fprintf (stderr, "Error: local header size != 16384 (%zd)\n",
+     	 sizeof(CLocalHeader));
       goto errcheck;
    }
 
@@ -1048,9 +1137,9 @@ void formatOperationSucessMsg(char *szMsg, int nMaxLen, char *szOperation)
   qwBytesPerMin = (QWORD) (((float)g_qwCopiedBytesCount) / fMinElapsed);
   
   snprintf(szMsg, nMaxLen, i18n("%s  successfully finished:\n\n"
-				"Time elapsed:...%s\n"
-				"Speed:..........%s/min\n"
-				"Data copied:....%s"),
+                                "Time elapsed: %s\n"
+                                "Speed: %s/min\n"
+                                "Data copied: %s"),
 	   szOperation,
 	   formatTime((DWORD)timeElapsed, szTemp1),
 	   formatSize(qwBytesPerMin, szTemp2),
