@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.86 $
+// $Revision: 1.2 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,6 +22,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#define __STDC_FORMAT_MACROS
+#include <inttypes.h>
 
 #ifdef HAVE_SYS_PARAM_H
   #include <sys/param.h>
@@ -88,8 +90,8 @@ void restoreMbr(char *szImageFile, COptions *options)
 
   CRestoreMbrWindow optGui;
   DWORD dwDiskCount;
-  DWORD dwOriginalMbrNb;
-  DWORD dwCurrentMbrNb;
+  uintptr_t dwOriginalMbrNb;
+  uintptr_t dwCurrentMbrNb;
   char szHdList[2048];
   char szTemp[512];
   char szTemp2[512];
@@ -209,7 +211,7 @@ void restoreMbr(char *szImageFile, COptions *options)
 	  image.closeReading(true);
           throw excep;
 	}
-      SNPRINTF(szTemp, "%.3lu: %s [%llu blocks]", i, mbrOriginal[i].szDevice, mbrOriginal[i].qwBlocksCount);
+      SNPRINTF(szTemp, "%.3u: %s [%"PRIu64" blocks]", i, mbrOriginal[i].szDevice, mbrOriginal[i].qwBlocksCount);
       if (options->bBatchMode == false)
 	optGui.addMbr(szTemp, i);
       
@@ -294,6 +296,8 @@ void restoreMbr(char *szImageFile, COptions *options)
 	nRes = fwrite(mbrOriginal[dwOriginalMbrNb].cData+MBR_SIZE_BOOT, MBR_SIZE_TABLE+2, 1, fDevice); // table + 0xAA55
     }
 
+  fflush(fDevice);
+
   // re-read partition table
   if (ioctl(fileno(fDevice), BLKRRPART))
     {
@@ -319,6 +323,28 @@ void restoreMbr(char *szImageFile, COptions *options)
   RETURN;
 }
 
+struct portable_tm *localtime_portable (const time_t *__timer)
+{
+static struct portable_tm ret;
+struct tm *host_tm;
+
+	host_tm=localtime (__timer);
+	ret.tm_sec=host_tm->tm_sec;
+	ret.tm_min=host_tm->tm_min;
+	ret.tm_hour=host_tm->tm_hour;
+	ret.tm_mday=host_tm->tm_mday;
+	ret.tm_mon=host_tm->tm_mon;
+	ret.tm_year=host_tm->tm_year;
+	ret.tm_wday=host_tm->tm_wday;
+	ret.tm_yday=host_tm->tm_yday;
+	ret.tm_isdst=host_tm->tm_isdst;
+
+	ret.tm_gmtoff=host_tm->tm_gmtoff;
+	ret.tm_zone=0;
+
+	return &ret;
+}
+
 // =======================================================
 void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
    COptions *options)
@@ -353,6 +379,8 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
   image.set_szPath(aux);
   
   //if (options->dwSplitMode == SPLIT_AUTO)
+  if ((strncmp(szImageName, "stdout", 6) == 0) || (strncmp(szImageName, "STDOUT", 6) == 0))
+	  options->qwSplitSize = 0;
   if (!options->qwSplitSize)
     {
       SNPRINTF(aux, "%s", szImageName); *(aux+MAXPATHLEN-1)='\0';
@@ -371,10 +399,10 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
       guiOpt.create(image.get_szImageFilename(), *options);
     }
 
-  g_interface -> StatusLine(i18n("initializing the operation. Please wait..."));
+    g_interface -> StatusLine(i18n("initializing the operation."));
   
   // ----------- system checks
-  nRes = systemChecks(szDevice, options->bFailIfMounted);
+  nRes = systemChecks(szDevice, options->bFailIfMounted, options->szFullyBatchMode);
   if (nRes == -1)
     THROW( ERR_FSCHK);
   
@@ -463,7 +491,7 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
   time(&dt);
 
   // current time/date (date of image creation)
-  headMain.dateCreate = *localtime(&dt);
+  headMain.dateCreate = *localtime_portable(&dt);
 
   // file format version
   strcpy(headMain.szVersion, CURRENT_IMAGE_FORMAT); 
@@ -511,7 +539,20 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
   //image.set_dwCurrentVolumeSize(0L);
   image.set_dwVolumeNumber(0L);
   //image.set_qwIdentificator(headMain.qwIdentificator);
-  
+
+  // gui = no
+  if (strlen(options->szFullyBatchMode)>0)
+  {
+                 showDebug(3,"szFullyBatchMode=%s\n, len=%d",options->szFullyBatchMode,strlen(options->szFullyBatchMode));
+    fprintf(stderr,"partimage: status: Partimage: %s\n", headMain.szVersion);
+    if (options->dwCompression == 0)
+      fprintf(stderr,"partimage: status: Image type: NONE\n");
+    else if (options->dwCompression == 1) +      fprintf(stderr,"partimage: status: Image type: GZIP\n");
+    else if (options->dwCompression == 2)
+      fprintf(stderr,"partimage: status: Image type: BZIP2\n");
+    else if (options->dwCompression == 3)
+      fprintf(stderr,"partimage: status: Image type: LZO\n");
+  }  
   try { image.write(&headMain, sizeof(CMainHeader), true); }
   catch ( CExceptions * excep )
     {
@@ -711,7 +752,7 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
   g_interface -> StatusLine(i18n("initializing the operation"));
   
   // ----------- system checks
-  nRes = systemChecks(szDevice, options->bFailIfMounted);
+  nRes = systemChecks(szDevice, options->bFailIfMounted, options->szFullyBatchMode);
   if (nRes == -1)
     THROW( ERR_FSCHK);
   
@@ -901,6 +942,22 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
   try { lockFile(fileno(fDeviceFile), szDevice); }
   catch ( CExceptions * excep )
     { throw excep; }
+  // gui = no
+  if (strlen(options->szFullyBatchMode)>0)
+  {
+          showDebug(3,"szFullyBatchMode=%s\n, len=%d",options->szFullyBatchMode,strlen(options->szFullyBatchMode));
+    options->dwCompression = (DWORD) image.getCompressionLevelForImage(szImageName);
+    fprintf(stderr,"partimage: status: Partimage: %s\n", headMain.szVersion);
+    if (options->dwCompression == 0)
+      fprintf(stderr,"partimage: status: Image type: NONE\n");
+    else if (options->dwCompression == 1)
+      fprintf(stderr,"partimage: status: Image type: GZIP\n");
+    else if (options->dwCompression == 2)
+      fprintf(stderr,"partimage: status: Image type: BZIP2\n");
+    else if (options->dwCompression == 3)
+      fprintf(stderr,"partimage: status: Image type: LZO\n");
+  }
+  g_interface -> StatusLine(i18n("Restoring partition from the image file..."));
 
 
   // --------- initialize info window -------------
@@ -916,7 +973,7 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
   
   // ----------- print informations about image file
   //guiInfo.showPartitionInfos(options->dwCompression, headMain.szOriginalDevice, headMain.szFileSystem, headMain.dateCreate, headMain.qwPartSize);
-  guiInfo.showImageFileInfo(szImageName, -1);
+//  guiInfo.showImageFileInfo(szImageName, -1, options->szFullyBatchMode);
 
   // -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
   CFSBase *FS = NULL; 
@@ -1181,6 +1238,7 @@ void showDebugOptions(COptions *options)
 	    "options.bBackupMBR = %d\n" 
 	    "options.bSimulateMode = %d\n" 
 	    "options.szAutoMount = %s\n" 
+            "options.szFullyBatchMode = %s\n"
 	    "============= end OPTIONS =====================\n\n\n",
 	    (int) options->bOverwrite, 
 	    //options->dwSplitMode,
@@ -1190,7 +1248,7 @@ void showDebugOptions(COptions *options)
 	    options->dwFinish, options->szServerName, 
 	    options->dwServerPort, (int) options->bBatchMode, (int)options->bSplitWait,
 	    (int)options->bSync, (int)options->bUseSSL, options->dwDebugLevel, 
-	    (int)options->bBackupMBR, (int)options->bSimulateMode, options->szAutoMount);
+            (int)options->bBackupMBR, (int)options->bSimulateMode, options->szAutoMount, options->szFullyBatchMode);
 }
 
 // =======================================================
@@ -1370,7 +1428,7 @@ int showUfsDisklabel(char *szDevice)
   FILE *fStream;
   char cBoot[512];
   CUFSDiskLabel dl;
-  int i, j;
+  int i=0, j;
   int nRes;
   CUFSPartition part;
   char letter='a';
@@ -1474,8 +1532,13 @@ int detectFileSystem(const char *szDevice, char *szFileSys)
 
   showDebug(4, "003 = after UFS\n");
 
-  // **** check for REISER file system [magic @ 65536]
-  nRes = CReiserPartdetect(cBuffer, szFileSys);
+  // **** check for REISER3 file system [magic @ 65536]
+  nRes = CReiser3Partdetect(cBuffer, szFileSys);
+  if (nRes > 0)
+    return nRes;
+
+ // **** check for REISER4 file system [magic @ 65536]
+  nRes = CReiser4Partdetect(cBuffer, szFileSys);
   if (nRes > 0)
     return nRes;
 
@@ -1622,7 +1685,7 @@ int getMajorMinor(char *szDevice, int *nMajor, int *nMinor)
 }
 
 // =======================================================
-int checkInodeForDevice(char *szDevice) // NEW FUNCTION
+int checkInodeForDevice(char *szDevice, char *szFullyBatchMode) // NEW FUNCTION
 {
   int nRes;
   struct stat fStat;
@@ -1636,11 +1699,13 @@ int checkInodeForDevice(char *szDevice) // NEW FUNCTION
   nRes = access(szDevice, F_OK);
   if (nRes == -1)
     {
-      nRes = g_interface->msgBoxYesNo(i18n("inode missing"), i18n("%s inode doesn't exists. Partimage can create it for you. You can also use the manual mknod "
+      if(strlen(szFullyBatchMode)<=0)
+      {
+          nRes = g_interface->msgBoxYesNo(i18n("inode missing"), i18n("%s inode doesn't exists. Partimage can create it for you. You can also use the manual mknod "
 								  "command. Do you want this inode to be created for you now ?"), szDevice);
-      if (nRes == MSGBOX_NO)
-	  RETURN_int(-1);
-
+          if (nRes == MSGBOX_NO)
+	      RETURN_int(-1);
+      }
       nRes = getMajorMinor(szDevice, &nMajor, &nMinor);
       if (nRes == -1)
 	RETURN_int(-1);
@@ -1864,13 +1929,13 @@ char *decodePartitionEntry(char *cPtr, int *nMajor, int *nMinor,
 }
 
 // =======================================================
-int systemChecks(char *szDevice, bool bFailIfMounted)
+int systemChecks(char *szDevice, bool bFailIfMounted, char *szFullyBatchMode)
 {
   char szMountPoint[1024];
   int nRes;
   
   // ----------- check for inode
-  nRes = checkInodeForDevice(szDevice);
+  nRes = checkInodeForDevice(szDevice, szFullyBatchMode);
   if (nRes == -1)
     RETURN_int(-1);	
   
@@ -1945,14 +2010,13 @@ int checkOptions(COptions options, char *szDevice, char *szImageFile)
 	  g_interface->msgBoxError(i18n("The third argument [%s] must be a valid regular file (not a directory, a symlink, ...)"), szImageFile);	
 	  RETURN_int(-1);
 	}
-
     }
 
   return 0;
 }
 
 // =======================================================
-int CReiserPartdetect(BYTE *cBootSect, char *szFileSystem)
+int CReiser3Partdetect(BYTE *cBootSect, char *szFileSystem)
 {
   BEGIN;
   
@@ -1975,6 +2039,23 @@ int CReiserPartdetect(BYTE *cBootSect, char *szFileSystem)
   RETURN_int(0); // Not a REISER partition
 }
 
+// =======================================================
+int CReiser4Partdetect(u8 *cBootSect, char *szFileSystem)
+{
+  BEGIN;
+  
+  if (memcmp(cBootSect+REISER4FS_DISK_OFFSET_IN_BYTES, REISER4FS_SUPER_MAGIC_STRING, strlen(REISER4FS_SUPER_MAGIC_STRING)) == 0)
+    {
+      strcpy(szFileSystem, "reiserfs-4.0");
+      return FS_REISER_4_0;
+    }
+  
+  showDebug(4, "001 = after check for Reiser-4.0\n");
+  
+
+  RETURN_int(0); // Not a REISER partition
+}
+
 // =======================================================
 int CExt2Partdetect(BYTE *cBootSect, char *szFileSystem)
 {
@@ -2272,3 +2353,231 @@ int CBefsPartdetect(BYTE *cBootSect, char *szFileSystem)
   strcpy(szFileSystem, "befs");
   RETURN_int(FS_BEFS);
 }
+
+static int str_begins_with(const char *s, const char *prefix)
+{
+  return strncmp(s, prefix, strlen(prefix)) == 0;
+}
+
+static char *sfdisk_line_to_disk_device(const char *line)
+{
+  static const char *new_disk_prefix = "# partition table of ";
+  if (str_begins_with(line, new_disk_prefix)) { 
+    const char *device = line + strlen(new_disk_prefix);
+    return strndup(device, strlen(device) - 1);
+  } else {
+    return NULL;
+  }
+}
+
+static char *sfdisk_line_to_partition_device(const char *line)
+{
+  if (char *p = strchr(line, ':')) {
+    while (p[-1] == ' ' && p > line) p--;
+    return strndup(line, p - line);
+  } else {
+    return NULL;
+  }
+}
+
+static char *full_device_to_device(char *device)
+{
+  char *p;
+  if (str_begins_with(device, "/dev/")) {
+    p = device + strlen("/dev/");
+  } else {
+    showDebug(2, "full_device_to_device: weird device %s\n", device);
+    p = strrchr(device, '/');
+    p = p ? p + 1 : device;
+  }
+  p = strdup(p);
+  char *q;
+  while ((q = strchr(p, '/'))!=NULL) *q = '_';
+
+  return p;
+}
+
+static void my_pclose(FILE *f)
+{
+  int status = pclose(f);
+  if (status != 0) showDebug(1, "ERROR: command failed (%d)\n", status);
+}
+
+static void write_unsigned(CImage *image, unsigned int nb)
+{
+  image->write((char *) &nb, sizeof(nb), true);
+}
+static void write_string(CImage *image, const char *s)
+{
+  unsigned int len = strlen(s);
+  write_unsigned(image, len);
+  image->write((void *) s, len, true);
+}
+static unsigned int read_unsigned(CImage *image)
+{
+  unsigned int nb;
+  image->read((char *) &nb, sizeof(nb), true);
+  return nb;
+}
+static char *read_string(CImage *image)
+{
+  unsigned int len = read_unsigned(image);
+  if (len == 0) return NULL;
+  char *s = (char *) malloc(len + 1);
+  image->read(s, len, true);
+  s[len] = '\0';
+  return s;
+}
+
+void save_all(char *szImageDir, COptions *options)
+{
+  BEGIN;  
+
+  char *sfdisk[1024];
+  unsigned int sfdisk_nb_lines = 0;
+  {
+    FILE *f = popen("LC_ALL=C sfdisk -d", "r");
+    char tmp[1024];
+  
+    while (1) {
+      if (!fgets(tmp, sizeof(tmp), f)) break;
+      if (sfdisk_nb_lines >= sizeof(sfdisk)) break;   
+      sfdisk[sfdisk_nb_lines++] = strdup(tmp);
+    }
+    my_pclose(f);
+  }
+
+  char aux[MAXPATHLEN];
+  CImage image(options);
+  SNPRINTF(aux, "%s/sfdisk.lst", szImageDir);
+  image.set_szImageFilename(aux);
+  image.openWriting();
+
+  write_unsigned(&image, sfdisk_nb_lines);
+
+  for (unsigned int i = 0; i < sfdisk_nb_lines; i++)
+    write_string(&image, sfdisk[i]);
+
+  for (unsigned int i = 0; i < sfdisk_nb_lines; i++)
+    if (char *device = sfdisk_line_to_disk_device(sfdisk[i])) {
+      showDebug(2, "opening %s to get its MBR\n", device);
+      FILE *f = fopen(device, "r");
+      char mbr[512];
+      if (f && fread(mbr, 512, 1, f) == 1) {
+	write_string(&image, device);
+	image.write(mbr, sizeof(mbr), true);
+      } else
+	showDebug(1, "ERROR: can't read MBR of device %s\n", device);
+      if (f) fclose(f);
+    }
+  write_unsigned(&image, 0);
+
+  closeFilesSave(false, *options, &image, NULL);
+
+  COptions save_part_options(*options);
+  save_part_options.qwSplitSize = 0;
+  save_part_options.bAskDesc = false;
+  save_part_options.bBackupMBR = false;
+  save_part_options.bCheckBeforeSaving = false; // checking always find an error, so...
+
+  for (unsigned int i = 0; i < sfdisk_nb_lines; i++) {
+    if (!strstr(sfdisk[i], "Id=83")) continue;
+
+    char *device = sfdisk_line_to_partition_device(sfdisk[i]);
+    if (!device) continue;
+
+    char *short_device = full_device_to_device(device);
+    SNPRINTF(aux, "%s/part_%s", szImageDir, short_device);
+    free(short_device);
+    showDebug(1, "savePartition(%s, %s, ...)\n", device, aux);
+    savePartition(device, aux, &save_part_options);
+
+    free(device);
+  }
+
+  RETURN;
+}
+
+void rest_all(char *szImageDir, COptions *options)
+{
+  BEGIN;  
+
+  char **sfdisk;
+  unsigned int sfdisk_nb_lines = 0;
+
+  char aux[MAXPATHLEN];
+  CImage image(options);
+
+  {
+    SNPRINTF(aux, "%s/sfdisk.lst", szImageDir);
+    image.set_szImageFilename(aux);
+    image.openReading();
+
+    sfdisk_nb_lines = read_unsigned(&image);
+    showDebug(2, "rest_all got sfdisk_nb_lines = %d\n", sfdisk_nb_lines);
+    sfdisk = (char **) alloca(sizeof(char *) * sfdisk_nb_lines);
+    for (unsigned int i = 0; i < sfdisk_nb_lines; i++) {
+      sfdisk[i] = read_string(&image);
+      showDebug(2, "rest_all got line = %s\n", sfdisk[i]);
+    }
+    while (1) {
+      char *device = read_string(&image);
+      if (!device) break;
+
+      char mbr[512];
+      image.read(mbr, 512, true);
+      
+      FILE *f = fopen(device, "r+b");
+      if (f && fwrite(mbr, 512, 1, f) == 1)
+	showDebug(1, "MBR of %s restored\n", device);
+      else
+	showDebug(1, "ERROR: restoring MBR of device %s failed: %s\n", device, strerror(errno));
+      if (f) fclose(f);
+    }
+    
+    // since sfdisk.lst is not really nice to the library, we can't close the thread from procReadBufferFromImage which has already exited
+    image.closeReading(false);
+  }
+
+  {
+    FILE *f = NULL;
+    for (unsigned int i = 0; i < sfdisk_nb_lines; i++) {
+      if (char *device = sfdisk_line_to_disk_device(sfdisk[i])) {
+	if (f) my_pclose(f);
+	SNPRINTF(aux, "sfdisk %s >>" PARTIMAGE_LOG " 2>>" PARTIMAGE_LOG, device);
+	showDebug(1, "calling %s\n", aux);
+	f = popen(aux, "w");
+	free(device);
+      }
+      if (f)
+	fputs(sfdisk[i], f);
+      else
+	showDebug(1, "dropping line: %s", sfdisk[i]);
+    }
+    if (f) my_pclose(f);
+  }
+
+  for (unsigned int i = 0; i < sfdisk_nb_lines; i++) {
+
+    char *device = sfdisk_line_to_partition_device(sfdisk[i]);
+    if (!device) continue;
+    
+    if (strstr(sfdisk[i], "Id=83")) {
+      char *short_device = full_device_to_device(device);
+      SNPRINTF(aux, "%s/part_%s", szImageDir, short_device);
+      free(short_device);
+      if (image.getImageDisk()->doesFileExists(aux)) {
+	showDebug(1, "restorePartition(%s, %s, ...)\n", device, aux);
+	restorePartition(device, aux, options);
+      } else
+	showDebug(1, "no such %s image disk, not restoring it\n", aux);
+    } else if (strstr(sfdisk[i], "Id=82")) {
+      SNPRINTF(aux, "mkswap %s >>" PARTIMAGE_LOG, device);
+      showDebug(2, "%s\n", aux);
+      system(aux);
+    }
+    free(device);
+  }
+  
+  RETURN;
+}
