@@ -149,6 +149,8 @@ unsigned int CNetServer::AcceptClient()
 
   if (strcmp(szBanner, szClientBanner))
     { // banners mismatch, client can't log on
+      showDebug(3, "wrong banner received from %s:%d\n", 
+         inet_ntoa(client_sa.sin_addr), ntohs(client_sa.sin_port));
       Send(client, "nack", 5);
 
 // return reason of refuse
@@ -161,6 +163,10 @@ unsigned int CNetServer::AcceptClient()
       else
         Send(client, "banners", 8);
       Release(client);
+      free(szBanner);
+      free(szClientBanner);
+      szBanner = NULL;
+      szClientBanner = NULL;
       THROW(ERR_REFUSED);
     }
 
@@ -168,6 +174,10 @@ unsigned int CNetServer::AcceptClient()
     { // partimaged support only MAX_CLIENTS at same time, user has to wait
       Send(client, "nack", 5);
       Send(client, "toomany", 8); 
+      free(szBanner);
+      free(szClientBanner);
+      szBanner = NULL;
+      szClientBanner = NULL;
       THROW(ERR_TOOMANY);
     }
   
@@ -199,6 +209,10 @@ unsigned int CNetServer::AcceptClient()
 
   Clients->Set(client, sock, client_sa, ssl); 
 
+  free(szBanner);
+  free(szClientBanner);
+  szBanner = NULL;
+  szClientBanner = NULL;
   showDebug(9, "end\n");
   return client;
 }
@@ -238,6 +252,7 @@ void CNetServer::SendExcep(unsigned int client, CExceptions * excep)
   dwAux = strlen(szAux)+1;
   Send(client, &dwAux, sizeof(dwAux));
   Send(client, szAux, dwAux);
+  delete mess;
 }
 
 char * CNetServer::GetPeer(unsigned int client)
@@ -268,7 +283,7 @@ unsigned int CNetServer::ValidatePass(unsigned int client)
 
   szSalt = GetSalt(szLogin);
   if (!szSalt)
-    szSalt = "";
+    szSalt = strdup("");
   nSize = strlen(szSalt)+1;
 
   nRes = Send(client, &nSize, sizeof(unsigned int));
@@ -278,7 +293,7 @@ unsigned int CNetServer::ValidatePass(unsigned int client)
   nRes = Recv(client, &nSize, sizeof(unsigned int)); 
   szPass = (char *) malloc (nSize);
   nRes = Recv(client, szPass, nSize); 
- 
+
   nRes = CheckAccess(g_bMustLogin, szLogin, szPass);
   showDebug(1, "return of checkaccess: %d\n", nRes);
   if (nRes)
@@ -288,6 +303,13 @@ unsigned int CNetServer::ValidatePass(unsigned int client)
     }
   else
     Send(client, " ack", 5);
+
+  free(szPass);
+  szPass = NULL;
+  free(szLogin);
+  szLogin = NULL;
+  free(szSalt);
+  szSalt = NULL;
   return nRes;
 }
   
