@@ -188,9 +188,10 @@ void CPartimagedInterfaceNewt::SetSize(int client, char * size)
 // =============================================================================
 void CPartimagedInterfaceNewt::Status(char * msg)
 {
-/*
+  char szBuf[1024];
   newtPopHelpLine();
-  newtPushHelpLine(msg);
+  snprintf(szBuf, sizeof(szBuf),
+    "%s [Press * to exit]", msg);
+  newtPushHelpLine(szBuf);
   newtRefresh();
-*/
 }
